

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (inaki@blueleaf.cc)

'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver' is a simple 'GI.Gio.Interfaces.ProxyResolver.ProxyResolver' implementation
that handles a single default proxy, multiple URI-scheme-specific
proxies, and a list of hosts that proxies should not be used for.

'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver' is never the default proxy resolver, but it
can be used as the base class for another proxy resolver
implementation, or it can be created and used manually, such as
with 'GI.Gio.Objects.SocketClient.socketClientSetProxyResolver'.

/Since: 2.36/
-}

#define ENABLE_OVERLOADING (MIN_VERSION_haskell_gi_overloading(1,0,0) \
       && !defined(__HADDOCK_VERSION__))

module GI.Gio.Objects.SimpleProxyResolver
    ( 

-- * Exported types
    SimpleProxyResolver(..)                 ,
    IsSimpleProxyResolver                   ,
    toSimpleProxyResolver                   ,
    noSimpleProxyResolver                   ,


 -- * Methods
-- ** new #method:new#

    simpleProxyResolverNew                  ,


-- ** setDefaultProxy #method:setDefaultProxy#

#if ENABLE_OVERLOADING
    SimpleProxyResolverSetDefaultProxyMethodInfo,
#endif
    simpleProxyResolverSetDefaultProxy      ,


-- ** setIgnoreHosts #method:setIgnoreHosts#

#if ENABLE_OVERLOADING
    SimpleProxyResolverSetIgnoreHostsMethodInfo,
#endif
    simpleProxyResolverSetIgnoreHosts       ,


-- ** setUriProxy #method:setUriProxy#

#if ENABLE_OVERLOADING
    SimpleProxyResolverSetUriProxyMethodInfo,
#endif
    simpleProxyResolverSetUriProxy          ,




 -- * Properties
-- ** defaultProxy #attr:defaultProxy#
{- | The default proxy URI that will be used for any URI that doesn\'t
match 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver':@/ignore-hosts/@, and doesn\'t match any
of the schemes set with 'GI.Gio.Objects.SimpleProxyResolver.simpleProxyResolverSetUriProxy'.

Note that as a special case, if this URI starts with
\"socks:\/\/\", 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver' will treat it as referring
to all three of the socks5, socks4a, and socks4 proxy types.
-}
#if ENABLE_OVERLOADING
    SimpleProxyResolverDefaultProxyPropertyInfo,
#endif
    constructSimpleProxyResolverDefaultProxy,
    getSimpleProxyResolverDefaultProxy      ,
    setSimpleProxyResolverDefaultProxy      ,
#if ENABLE_OVERLOADING
    simpleProxyResolverDefaultProxy         ,
#endif


-- ** ignoreHosts #attr:ignoreHosts#
{- | A list of hostnames and IP addresses that the resolver should
allow direct connections to.

Entries can be in one of 4 formats:

* A hostname, such as \"example.com\", \".example.com\", or
\"*.example.com\", any of which match \"example.com\" or
any subdomain of it.
* An IPv4 or IPv6 address, such as \"192.168.1.1\",
which matches only that address.
* A hostname or IP address followed by a port, such as
\"example.com:80\", which matches whatever the hostname or IP
address would match, but only for URLs with the (explicitly)
indicated port. In the case of an IPv6 address, the address
part must appear in brackets: \"[::1]:443\"
* An IP address range, given by a base address and prefix length,
such as \"fe80::\/10\", which matches any address in that range.


Note that when dealing with Unicode hostnames, the matching is
done against the ASCII form of the name.

Also note that hostname exclusions apply only to connections made
to hosts identified by name, and IP address exclusions apply only
to connections made to hosts identified by address. That is, if
example.com has an address of 192.168.1.1, and the :ignore-hosts list
contains only \"192.168.1.1\", then a connection to \"example.com\"
(eg, via a 'GI.Gio.Objects.NetworkAddress.NetworkAddress') will use the proxy, and a connection to
\"192.168.1.1\" (eg, via a 'GI.Gio.Objects.InetSocketAddress.InetSocketAddress') will not.

These rules match the \"ignore-hosts\"\/\"noproxy\" rules most
commonly used by other applications.
-}
#if ENABLE_OVERLOADING
    SimpleProxyResolverIgnoreHostsPropertyInfo,
#endif
    clearSimpleProxyResolverIgnoreHosts     ,
    constructSimpleProxyResolverIgnoreHosts ,
    getSimpleProxyResolverIgnoreHosts       ,
    setSimpleProxyResolverIgnoreHosts       ,
#if ENABLE_OVERLOADING
    simpleProxyResolverIgnoreHosts          ,
#endif




    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL

import qualified GI.GObject.Objects.Object as GObject.Object
import {-# SOURCE #-} qualified GI.Gio.Interfaces.ProxyResolver as Gio.ProxyResolver

-- | Memory-managed wrapper type.
newtype SimpleProxyResolver = SimpleProxyResolver (ManagedPtr SimpleProxyResolver)
foreign import ccall "g_simple_proxy_resolver_get_type"
    c_g_simple_proxy_resolver_get_type :: IO GType

instance GObject SimpleProxyResolver where
    gobjectType = c_g_simple_proxy_resolver_get_type
    

-- | Type class for types which can be safely cast to `SimpleProxyResolver`, for instance with `toSimpleProxyResolver`.
class (GObject o, O.IsDescendantOf SimpleProxyResolver o) => IsSimpleProxyResolver o
instance (GObject o, O.IsDescendantOf SimpleProxyResolver o) => IsSimpleProxyResolver o

instance O.HasParentTypes SimpleProxyResolver
type instance O.ParentTypes SimpleProxyResolver = '[GObject.Object.Object, Gio.ProxyResolver.ProxyResolver]

-- | Cast to `SimpleProxyResolver`, for types for which this is known to be safe. For general casts, use `Data.GI.Base.ManagedPtr.castTo`.
toSimpleProxyResolver :: (MonadIO m, IsSimpleProxyResolver o) => o -> m SimpleProxyResolver
toSimpleProxyResolver = liftIO . unsafeCastTo SimpleProxyResolver

-- | A convenience alias for `Nothing` :: `Maybe` `SimpleProxyResolver`.
noSimpleProxyResolver :: Maybe SimpleProxyResolver
noSimpleProxyResolver = Nothing

#if ENABLE_OVERLOADING
type family ResolveSimpleProxyResolverMethod (t :: Symbol) (o :: *) :: * where
    ResolveSimpleProxyResolverMethod "bindProperty" o = GObject.Object.ObjectBindPropertyMethodInfo
    ResolveSimpleProxyResolverMethod "bindPropertyFull" o = GObject.Object.ObjectBindPropertyFullMethodInfo
    ResolveSimpleProxyResolverMethod "forceFloating" o = GObject.Object.ObjectForceFloatingMethodInfo
    ResolveSimpleProxyResolverMethod "freezeNotify" o = GObject.Object.ObjectFreezeNotifyMethodInfo
    ResolveSimpleProxyResolverMethod "getv" o = GObject.Object.ObjectGetvMethodInfo
    ResolveSimpleProxyResolverMethod "isFloating" o = GObject.Object.ObjectIsFloatingMethodInfo
    ResolveSimpleProxyResolverMethod "isSupported" o = Gio.ProxyResolver.ProxyResolverIsSupportedMethodInfo
    ResolveSimpleProxyResolverMethod "lookup" o = Gio.ProxyResolver.ProxyResolverLookupMethodInfo
    ResolveSimpleProxyResolverMethod "lookupAsync" o = Gio.ProxyResolver.ProxyResolverLookupAsyncMethodInfo
    ResolveSimpleProxyResolverMethod "lookupFinish" o = Gio.ProxyResolver.ProxyResolverLookupFinishMethodInfo
    ResolveSimpleProxyResolverMethod "notify" o = GObject.Object.ObjectNotifyMethodInfo
    ResolveSimpleProxyResolverMethod "notifyByPspec" o = GObject.Object.ObjectNotifyByPspecMethodInfo
    ResolveSimpleProxyResolverMethod "ref" o = GObject.Object.ObjectRefMethodInfo
    ResolveSimpleProxyResolverMethod "refSink" o = GObject.Object.ObjectRefSinkMethodInfo
    ResolveSimpleProxyResolverMethod "runDispose" o = GObject.Object.ObjectRunDisposeMethodInfo
    ResolveSimpleProxyResolverMethod "stealData" o = GObject.Object.ObjectStealDataMethodInfo
    ResolveSimpleProxyResolverMethod "stealQdata" o = GObject.Object.ObjectStealQdataMethodInfo
    ResolveSimpleProxyResolverMethod "thawNotify" o = GObject.Object.ObjectThawNotifyMethodInfo
    ResolveSimpleProxyResolverMethod "unref" o = GObject.Object.ObjectUnrefMethodInfo
    ResolveSimpleProxyResolverMethod "watchClosure" o = GObject.Object.ObjectWatchClosureMethodInfo
    ResolveSimpleProxyResolverMethod "getData" o = GObject.Object.ObjectGetDataMethodInfo
    ResolveSimpleProxyResolverMethod "getProperty" o = GObject.Object.ObjectGetPropertyMethodInfo
    ResolveSimpleProxyResolverMethod "getQdata" o = GObject.Object.ObjectGetQdataMethodInfo
    ResolveSimpleProxyResolverMethod "setData" o = GObject.Object.ObjectSetDataMethodInfo
    ResolveSimpleProxyResolverMethod "setDefaultProxy" o = SimpleProxyResolverSetDefaultProxyMethodInfo
    ResolveSimpleProxyResolverMethod "setIgnoreHosts" o = SimpleProxyResolverSetIgnoreHostsMethodInfo
    ResolveSimpleProxyResolverMethod "setProperty" o = GObject.Object.ObjectSetPropertyMethodInfo
    ResolveSimpleProxyResolverMethod "setUriProxy" o = SimpleProxyResolverSetUriProxyMethodInfo
    ResolveSimpleProxyResolverMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveSimpleProxyResolverMethod t SimpleProxyResolver, O.MethodInfo info SimpleProxyResolver p) => OL.IsLabel t (SimpleProxyResolver -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#else
    fromLabel _ = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#endif

#endif

-- VVV Prop "default-proxy"
   -- Type: TBasicType TUTF8
   -- Flags: [PropertyReadable,PropertyWritable]
   -- Nullable: (Nothing,Just False)

{- |
Get the value of the “@default-proxy@” property.
When <https://github.com/haskell-gi/haskell-gi/wiki/Overloading overloading> is enabled, this is equivalent to

@
'Data.GI.Base.Attributes.get' simpleProxyResolver #defaultProxy
@
-}
getSimpleProxyResolverDefaultProxy :: (MonadIO m, IsSimpleProxyResolver o) => o -> m (Maybe T.Text)
getSimpleProxyResolverDefaultProxy obj = liftIO $ B.Properties.getObjectPropertyString obj "default-proxy"

{- |
Set the value of the “@default-proxy@” property.
When <https://github.com/haskell-gi/haskell-gi/wiki/Overloading overloading> is enabled, this is equivalent to

@
'Data.GI.Base.Attributes.set' simpleProxyResolver [ #defaultProxy 'Data.GI.Base.Attributes.:=' value ]
@
-}
setSimpleProxyResolverDefaultProxy :: (MonadIO m, IsSimpleProxyResolver o) => o -> T.Text -> m ()
setSimpleProxyResolverDefaultProxy obj val = liftIO $ B.Properties.setObjectPropertyString obj "default-proxy" (Just val)

{- |
Construct a `GValueConstruct` with valid value for the “@default-proxy@” property. This is rarely needed directly, but it is used by `Data.GI.Base.Constructible.new`.
-}
constructSimpleProxyResolverDefaultProxy :: (IsSimpleProxyResolver o) => T.Text -> IO (GValueConstruct o)
constructSimpleProxyResolverDefaultProxy val = B.Properties.constructObjectPropertyString "default-proxy" (Just val)

#if ENABLE_OVERLOADING
data SimpleProxyResolverDefaultProxyPropertyInfo
instance AttrInfo SimpleProxyResolverDefaultProxyPropertyInfo where
    type AttrAllowedOps SimpleProxyResolverDefaultProxyPropertyInfo = '[ 'AttrSet, 'AttrConstruct, 'AttrGet]
    type AttrSetTypeConstraint SimpleProxyResolverDefaultProxyPropertyInfo = (~) T.Text
    type AttrBaseTypeConstraint SimpleProxyResolverDefaultProxyPropertyInfo = IsSimpleProxyResolver
    type AttrGetType SimpleProxyResolverDefaultProxyPropertyInfo = (Maybe T.Text)
    type AttrLabel SimpleProxyResolverDefaultProxyPropertyInfo = "default-proxy"
    type AttrOrigin SimpleProxyResolverDefaultProxyPropertyInfo = SimpleProxyResolver
    attrGet _ = getSimpleProxyResolverDefaultProxy
    attrSet _ = setSimpleProxyResolverDefaultProxy
    attrConstruct _ = constructSimpleProxyResolverDefaultProxy
    attrClear _ = undefined
#endif

-- VVV Prop "ignore-hosts"
   -- Type: TCArray True (-1) (-1) (TBasicType TUTF8)
   -- Flags: [PropertyReadable,PropertyWritable]
   -- Nullable: (Nothing,Nothing)

{- |
Get the value of the “@ignore-hosts@” property.
When <https://github.com/haskell-gi/haskell-gi/wiki/Overloading overloading> is enabled, this is equivalent to

@
'Data.GI.Base.Attributes.get' simpleProxyResolver #ignoreHosts
@
-}
getSimpleProxyResolverIgnoreHosts :: (MonadIO m, IsSimpleProxyResolver o) => o -> m (Maybe [T.Text])
getSimpleProxyResolverIgnoreHosts obj = liftIO $ B.Properties.getObjectPropertyStringArray obj "ignore-hosts"

{- |
Set the value of the “@ignore-hosts@” property.
When <https://github.com/haskell-gi/haskell-gi/wiki/Overloading overloading> is enabled, this is equivalent to

@
'Data.GI.Base.Attributes.set' simpleProxyResolver [ #ignoreHosts 'Data.GI.Base.Attributes.:=' value ]
@
-}
setSimpleProxyResolverIgnoreHosts :: (MonadIO m, IsSimpleProxyResolver o) => o -> [T.Text] -> m ()
setSimpleProxyResolverIgnoreHosts obj val = liftIO $ B.Properties.setObjectPropertyStringArray obj "ignore-hosts" (Just val)

{- |
Construct a `GValueConstruct` with valid value for the “@ignore-hosts@” property. This is rarely needed directly, but it is used by `Data.GI.Base.Constructible.new`.
-}
constructSimpleProxyResolverIgnoreHosts :: (IsSimpleProxyResolver o) => [T.Text] -> IO (GValueConstruct o)
constructSimpleProxyResolverIgnoreHosts val = B.Properties.constructObjectPropertyStringArray "ignore-hosts" (Just val)

{- |
Set the value of the “@ignore-hosts@” property to `Nothing`.
When <https://github.com/haskell-gi/haskell-gi/wiki/Overloading overloading> is enabled, this is equivalent to

@
'Data.GI.Base.Attributes.clear' #ignoreHosts
@
-}
clearSimpleProxyResolverIgnoreHosts :: (MonadIO m, IsSimpleProxyResolver o) => o -> m ()
clearSimpleProxyResolverIgnoreHosts obj = liftIO $ B.Properties.setObjectPropertyStringArray obj "ignore-hosts" (Nothing :: Maybe [T.Text])

#if ENABLE_OVERLOADING
data SimpleProxyResolverIgnoreHostsPropertyInfo
instance AttrInfo SimpleProxyResolverIgnoreHostsPropertyInfo where
    type AttrAllowedOps SimpleProxyResolverIgnoreHostsPropertyInfo = '[ 'AttrSet, 'AttrConstruct, 'AttrGet, 'AttrClear]
    type AttrSetTypeConstraint SimpleProxyResolverIgnoreHostsPropertyInfo = (~) [T.Text]
    type AttrBaseTypeConstraint SimpleProxyResolverIgnoreHostsPropertyInfo = IsSimpleProxyResolver
    type AttrGetType SimpleProxyResolverIgnoreHostsPropertyInfo = (Maybe [T.Text])
    type AttrLabel SimpleProxyResolverIgnoreHostsPropertyInfo = "ignore-hosts"
    type AttrOrigin SimpleProxyResolverIgnoreHostsPropertyInfo = SimpleProxyResolver
    attrGet _ = getSimpleProxyResolverIgnoreHosts
    attrSet _ = setSimpleProxyResolverIgnoreHosts
    attrConstruct _ = constructSimpleProxyResolverIgnoreHosts
    attrClear _ = clearSimpleProxyResolverIgnoreHosts
#endif

#if ENABLE_OVERLOADING
instance O.HasAttributeList SimpleProxyResolver
type instance O.AttributeList SimpleProxyResolver = SimpleProxyResolverAttributeList
type SimpleProxyResolverAttributeList = ('[ '("defaultProxy", SimpleProxyResolverDefaultProxyPropertyInfo), '("ignoreHosts", SimpleProxyResolverIgnoreHostsPropertyInfo)] :: [(Symbol, *)])
#endif

#if ENABLE_OVERLOADING
simpleProxyResolverDefaultProxy :: AttrLabelProxy "defaultProxy"
simpleProxyResolverDefaultProxy = AttrLabelProxy

simpleProxyResolverIgnoreHosts :: AttrLabelProxy "ignoreHosts"
simpleProxyResolverIgnoreHosts = AttrLabelProxy

#endif

#if ENABLE_OVERLOADING
type instance O.SignalList SimpleProxyResolver = SimpleProxyResolverSignalList
type SimpleProxyResolverSignalList = ('[ '("notify", GObject.Object.ObjectNotifySignalInfo)] :: [(Symbol, *)])

#endif

-- method SimpleProxyResolver::set_default_proxy
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "resolver", argType = TInterface (Name {namespace = "Gio", name = "SimpleProxyResolver"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GSimpleProxyResolver", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "default_proxy", argType = TBasicType TUTF8, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "the default proxy to use", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Nothing
-- throws : False
-- Skip return : False

foreign import ccall "g_simple_proxy_resolver_set_default_proxy" g_simple_proxy_resolver_set_default_proxy :: 
    Ptr SimpleProxyResolver ->              -- resolver : TInterface (Name {namespace = "Gio", name = "SimpleProxyResolver"})
    CString ->                              -- default_proxy : TBasicType TUTF8
    IO ()

{- |
Sets the default proxy on /@resolver@/, to be used for any URIs that
don\'t match 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver':@/ignore-hosts/@ or a proxy set
via 'GI.Gio.Objects.SimpleProxyResolver.simpleProxyResolverSetUriProxy'.

If /@defaultProxy@/ starts with \"socks:\/\/\",
'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver' will treat it as referring to all three of
the socks5, socks4a, and socks4 proxy types.

/Since: 2.36/
-}
simpleProxyResolverSetDefaultProxy ::
    (B.CallStack.HasCallStack, MonadIO m, IsSimpleProxyResolver a) =>
    a
    {- ^ /@resolver@/: a 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver' -}
    -> T.Text
    {- ^ /@defaultProxy@/: the default proxy to use -}
    -> m ()
simpleProxyResolverSetDefaultProxy resolver defaultProxy = liftIO $ do
    resolver' <- unsafeManagedPtrCastPtr resolver
    defaultProxy' <- textToCString defaultProxy
    g_simple_proxy_resolver_set_default_proxy resolver' defaultProxy'
    touchManagedPtr resolver
    freeMem defaultProxy'
    return ()

#if ENABLE_OVERLOADING
data SimpleProxyResolverSetDefaultProxyMethodInfo
instance (signature ~ (T.Text -> m ()), MonadIO m, IsSimpleProxyResolver a) => O.MethodInfo SimpleProxyResolverSetDefaultProxyMethodInfo a signature where
    overloadedMethod _ = simpleProxyResolverSetDefaultProxy

#endif

-- method SimpleProxyResolver::set_ignore_hosts
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "resolver", argType = TInterface (Name {namespace = "Gio", name = "SimpleProxyResolver"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GSimpleProxyResolver", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "ignore_hosts", argType = TBasicType TUTF8, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "%NULL-terminated list of hosts/IP addresses\n    to not use a proxy for", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Nothing
-- throws : False
-- Skip return : False

foreign import ccall "g_simple_proxy_resolver_set_ignore_hosts" g_simple_proxy_resolver_set_ignore_hosts :: 
    Ptr SimpleProxyResolver ->              -- resolver : TInterface (Name {namespace = "Gio", name = "SimpleProxyResolver"})
    CString ->                              -- ignore_hosts : TBasicType TUTF8
    IO ()

{- |
Sets the list of ignored hosts.

See 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver':@/ignore-hosts/@ for more details on how the
/@ignoreHosts@/ argument is interpreted.

/Since: 2.36/
-}
simpleProxyResolverSetIgnoreHosts ::
    (B.CallStack.HasCallStack, MonadIO m, IsSimpleProxyResolver a) =>
    a
    {- ^ /@resolver@/: a 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver' -}
    -> T.Text
    {- ^ /@ignoreHosts@/: 'Nothing'-terminated list of hosts\/IP addresses
    to not use a proxy for -}
    -> m ()
simpleProxyResolverSetIgnoreHosts resolver ignoreHosts = liftIO $ do
    resolver' <- unsafeManagedPtrCastPtr resolver
    ignoreHosts' <- textToCString ignoreHosts
    g_simple_proxy_resolver_set_ignore_hosts resolver' ignoreHosts'
    touchManagedPtr resolver
    freeMem ignoreHosts'
    return ()

#if ENABLE_OVERLOADING
data SimpleProxyResolverSetIgnoreHostsMethodInfo
instance (signature ~ (T.Text -> m ()), MonadIO m, IsSimpleProxyResolver a) => O.MethodInfo SimpleProxyResolverSetIgnoreHostsMethodInfo a signature where
    overloadedMethod _ = simpleProxyResolverSetIgnoreHosts

#endif

-- method SimpleProxyResolver::set_uri_proxy
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "resolver", argType = TInterface (Name {namespace = "Gio", name = "SimpleProxyResolver"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GSimpleProxyResolver", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "uri_scheme", argType = TBasicType TUTF8, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "the URI scheme to add a proxy for", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "proxy", argType = TBasicType TUTF8, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "the proxy to use for @uri_scheme", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Nothing
-- throws : False
-- Skip return : False

foreign import ccall "g_simple_proxy_resolver_set_uri_proxy" g_simple_proxy_resolver_set_uri_proxy :: 
    Ptr SimpleProxyResolver ->              -- resolver : TInterface (Name {namespace = "Gio", name = "SimpleProxyResolver"})
    CString ->                              -- uri_scheme : TBasicType TUTF8
    CString ->                              -- proxy : TBasicType TUTF8
    IO ()

{- |
Adds a URI-scheme-specific proxy to /@resolver@/; URIs whose scheme
matches /@uriScheme@/ (and which don\'t match
'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver':@/ignore-hosts/@) will be proxied via /@proxy@/.

As with 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver':@/default-proxy/@, if /@proxy@/ starts with
\"socks:\/\/\", 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver' will treat it
as referring to all three of the socks5, socks4a, and socks4 proxy
types.

/Since: 2.36/
-}
simpleProxyResolverSetUriProxy ::
    (B.CallStack.HasCallStack, MonadIO m, IsSimpleProxyResolver a) =>
    a
    {- ^ /@resolver@/: a 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver' -}
    -> T.Text
    {- ^ /@uriScheme@/: the URI scheme to add a proxy for -}
    -> T.Text
    {- ^ /@proxy@/: the proxy to use for /@uriScheme@/ -}
    -> m ()
simpleProxyResolverSetUriProxy resolver uriScheme proxy = liftIO $ do
    resolver' <- unsafeManagedPtrCastPtr resolver
    uriScheme' <- textToCString uriScheme
    proxy' <- textToCString proxy
    g_simple_proxy_resolver_set_uri_proxy resolver' uriScheme' proxy'
    touchManagedPtr resolver
    freeMem uriScheme'
    freeMem proxy'
    return ()

#if ENABLE_OVERLOADING
data SimpleProxyResolverSetUriProxyMethodInfo
instance (signature ~ (T.Text -> T.Text -> m ()), MonadIO m, IsSimpleProxyResolver a) => O.MethodInfo SimpleProxyResolverSetUriProxyMethodInfo a signature where
    overloadedMethod _ = simpleProxyResolverSetUriProxy

#endif

-- method SimpleProxyResolver::new
-- method type : MemberFunction
-- Args : [Arg {argCName = "default_proxy", argType = TBasicType TUTF8, direction = DirectionIn, mayBeNull = True, argDoc = Documentation {rawDocText = Just "the default proxy to use, eg\n    \"socks://192.168.1.1\"", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "ignore_hosts", argType = TBasicType TUTF8, direction = DirectionIn, mayBeNull = True, argDoc = Documentation {rawDocText = Just "an optional list of hosts/IP addresses\n    to not use a proxy for.", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Just (TInterface (Name {namespace = "Gio", name = "ProxyResolver"}))
-- throws : False
-- Skip return : False

foreign import ccall "g_simple_proxy_resolver_new" g_simple_proxy_resolver_new :: 
    CString ->                              -- default_proxy : TBasicType TUTF8
    CString ->                              -- ignore_hosts : TBasicType TUTF8
    IO (Ptr Gio.ProxyResolver.ProxyResolver)

{- |
Creates a new 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver'. See
'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver':@/default-proxy/@ and
'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver':@/ignore-hosts/@ for more details on how the
arguments are interpreted.

/Since: 2.36/
-}
simpleProxyResolverNew ::
    (B.CallStack.HasCallStack, MonadIO m) =>
    Maybe (T.Text)
    {- ^ /@defaultProxy@/: the default proxy to use, eg
    \"socks:\/\/192.168.1.1\" -}
    -> Maybe (T.Text)
    {- ^ /@ignoreHosts@/: an optional list of hosts\/IP addresses
    to not use a proxy for. -}
    -> m Gio.ProxyResolver.ProxyResolver
    {- ^ __Returns:__ a new 'GI.Gio.Objects.SimpleProxyResolver.SimpleProxyResolver' -}
simpleProxyResolverNew defaultProxy ignoreHosts = liftIO $ do
    maybeDefaultProxy <- case defaultProxy of
        Nothing -> return nullPtr
        Just jDefaultProxy -> do
            jDefaultProxy' <- textToCString jDefaultProxy
            return jDefaultProxy'
    maybeIgnoreHosts <- case ignoreHosts of
        Nothing -> return nullPtr
        Just jIgnoreHosts -> do
            jIgnoreHosts' <- textToCString jIgnoreHosts
            return jIgnoreHosts'
    result <- g_simple_proxy_resolver_new maybeDefaultProxy maybeIgnoreHosts
    checkUnexpectedReturnNULL "simpleProxyResolverNew" result
    result' <- (wrapObject Gio.ProxyResolver.ProxyResolver) result
    freeMem maybeDefaultProxy
    freeMem maybeIgnoreHosts
    return result'

#if ENABLE_OVERLOADING
#endif


